@@ -24,46 +24,82 @@ from __future__ import absolute_import, print_function
 
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GObject
-GObject.threads_init()
 
 import warnings
 warnings.filterwarnings(
     "ignore", "Accessed deprecated property", DeprecationWarning)
 
+import logging
 import dbus
 import os
-import subprocess
-import sys
-from .SimpleGtkbuilderApp import SimpleGtkbuilderApp
 
 from gettext import gettext as _
 
 
-class Dialog(SimpleGtkbuilderApp):
+class Dialog(object):
     def __init__(self, window_main):
         self.window_main = window_main
-        self.focus_button = None
-        SimpleGtkbuilderApp.__init__(
-            self,
-            self.window_main.datadir + "/gtkbuilder/Dialog.ui",
-            "update-manager")
 
-    def main(self):
-        self.window_main.push(self.pane_dialog, self)
-        if self.focus_button:
-            self.focus_button.grab_focus()
+    def start(self):
+        pass
+
+    def close(self):
+        return self.stop()
+
+    def stop(self):
+        pass
 
     def run(self, parent=None):
-        if self.focus_button:
-            self.focus_button.grab_focus()
+        pass
+
+
+class BuilderDialog(Dialog, Gtk.Alignment):
+    def __init__(self, window_main, ui_path, root_widget):
+        Dialog.__init__(self, window_main)
+        Gtk.Alignment.__init__(self)
+
+        builder = self._load_ui(ui_path, root_widget)
+        self.add(builder.get_object(root_widget))
+        self.show()
+
+    def _load_ui(self, path, root_widget, domain="update-manager"):
+        builder = Gtk.Builder()
+        builder.set_translation_domain(domain)
+        builder.add_objects_from_file(path, [root_widget])
+        builder.connect_signals(self)
+
+        for o in builder.get_objects():
+            if issubclass(type(o), Gtk.Buildable):
+                name = Gtk.Buildable.get_name(o)
+                setattr(self, name, o)
+            else:
+                logging.debug("WARNING: can not get name for '%s'" % o)
+
+        return builder
+
+    def run(self, parent=None):
+        # FIXME: THIS WILL CRASH!
         if parent:
             self.window_dialog.set_transient_for(parent)
             self.window_dialog.set_modal(True)
         self.window_dialog.run()
 
-    def close(self):
-        sys.exit(0)  # By default, exit the app
+
+class InternalDialog(BuilderDialog):
+    def __init__(self, window_main, content_widget=None):
+        ui_path = os.path.join(window_main.datadir, "gtkbuilder/Dialog.ui")
+        BuilderDialog.__init__(self, window_main, ui_path, "pane_dialog")
+
+        self.focus_button = None
+        self.set_content_widget(content_widget)
+        self.connect("realize", self._on_realize)
+
+    def _on_realize(self, user_data):
+        if self.focus_button:
+            self.focus_button.set_can_default(True)
+            self.focus_button.set_can_focus(True)
+            self.focus_button.grab_default()
+            self.focus_button.grab_focus()
 
     def add_button(self, label, callback, secondary=False):
         # from_stock tries stock first and falls back to mnemonic
@@ -83,48 +119,53 @@ class Dialog(SimpleGtkbuilderApp):
             return None
 
     def on_settings_button_clicked(self):
-        cmd = ["/usr/bin/software-properties-gtk",
-               "--open-tab", "2",
-               # FIXME: once get_xid() is available via introspections, add
-               #        this back
-               #"--toplevel", "%s" % self.window_main.get_window().get_xid()
-               ]
-        subprocess.Popen(cmd)
+        self.window_main.show_settings()
 
     def set_header(self, label):
-        self.label_header.set_label(label)
+        if label:
+            self.label_header.set_markup(
+                "<span size='larger' weight='bold'>%s</span>" % label)
+        self.label_header.set_visible(bool(label))
 
     def set_desc(self, label):
-        self.label_desc.set_label(label)
+        if label:
+            self.label_desc.set_markup(label)
         self.label_desc.set_visible(bool(label))
 
+    def set_content_widget(self, content_widget):
+        if content_widget:
+            self.main_container.add(content_widget)
+        self.main_container.set_visible(bool(content_widget))
 
-class StoppedUpdatesDialog(Dialog):
+
+class StoppedUpdatesDialog(InternalDialog):
     def __init__(self, window_main):
-        Dialog.__init__(self, window_main)
+        InternalDialog.__init__(self, window_main)
         self.set_header(_("You stopped the check for updates."))
         self.add_settings_button()
         self.add_button(_("_Check Again"), self.check)
-        self.focus_button = self.add_button(Gtk.STOCK_OK, self.close)
+        self.focus_button = self.add_button(Gtk.STOCK_OK,
+                                            self.window_main.close)
 
     def check(self):
         self.window_main.start_update()
 
 
-class NoUpdatesDialog(Dialog):
+class NoUpdatesDialog(InternalDialog):
     def __init__(self, window_main, error_occurred=False):
-        Dialog.__init__(self, window_main)
+        InternalDialog.__init__(self, window_main)
         if error_occurred:
             self.set_header(_("No software updates are available."))
         else:
             self.set_header(_("The software on this computer is up to date."))
         self.add_settings_button()
-        self.focus_button = self.add_button(Gtk.STOCK_OK, self.close)
+        self.focus_button = self.add_button(Gtk.STOCK_OK,
+                                            self.window_main.close)
 
 
-class DistUpgradeDialog(Dialog):
+class DistUpgradeDialog(InternalDialog):
     def __init__(self, window_main, meta_release):
-        Dialog.__init__(self, window_main)
+        InternalDialog.__init__(self, window_main)
         self.meta_release = meta_release
         self.set_header(_("The software on this computer is up to date."))
         # Translators: these are Ubuntu version names like "Ubuntu 12.04"
@@ -134,7 +175,8 @@ class DistUpgradeDialog(Dialog):
                       meta_release.current_dist_version))
         self.add_settings_button()
         self.add_button(_("Upgrade…"), self.upgrade)
-        self.focus_button = self.add_button(Gtk.STOCK_OK, self.close)
+        self.focus_button = self.add_button(Gtk.STOCK_OK,
+                                            self.window_main.close)
 
     def upgrade(self):
         # Pass on several arguments
@@ -169,9 +211,9 @@ class UnsupportedDialog(DistUpgradeDialog):
         DistUpgradeDialog.run(self, parent)
 
 
-class PartialUpgradeDialog(Dialog):
+class PartialUpgradeDialog(InternalDialog):
     def __init__(self, window_main):
-        Dialog.__init__(self, window_main)
+        InternalDialog.__init__(self, window_main)
         self.set_header(_("Not all updates can be installed"))
         self.set_desc(_(
             """Run a partial upgrade, to install as many updates as possible.
@@ -191,24 +233,25 @@ class PartialUpgradeDialog(Dialog):
                  "/usr/lib/ubuntu-release-upgrader/do-partial-upgrade "
                  "--frontend=DistUpgradeViewGtk3")
 
-    def main(self):
-        Dialog.main(self)
+    def start(self):
+        Dialog.start(self)
         # Block progress until user has answered this question
         Gtk.main()
 
 
-class ErrorDialog(Dialog):
+class ErrorDialog(InternalDialog):
     def __init__(self, window_main, header, desc=None):
-        Dialog.__init__(self, window_main)
+        InternalDialog.__init__(self, window_main)
         self.set_header(header)
         if desc:
             self.set_desc(desc)
             self.label_desc.set_selectable(True)
         self.add_settings_button()
-        self.focus_button = self.add_button(Gtk.STOCK_OK, self.close)
+        self.focus_button = self.add_button(Gtk.STOCK_OK,
+                                            self.window_main.close)
 
-    def main(self):
-        Dialog.main(self)
+    def start(self):
+        Dialog.start(self)
         # The label likes to start selecting everything (b/c it got focus
         # before we switched to our default button).
         self.label_desc.select_region(0, 0)
@@ -229,15 +272,15 @@ class UpdateErrorDialog(ErrorDialog):
         self.window_main.start_available(error_occurred=True)
 
 
-class NeedRestartDialog(Dialog):
+class NeedRestartDialog(InternalDialog):
     def __init__(self, window_main):
-        Dialog.__init__(self, window_main)
+        InternalDialog.__init__(self, window_main)
         self.set_header(
             _("The computer needs to restart to finish installing updates."))
         self.focus_button = self.add_button(_("_Restart"), self.restart)
 
-    def main(self):
-        Dialog.main(self)
+    def start(self):
+        Dialog.start(self)
         # Turn off close button
         self.window_main.realize()
         self.window_main.get_window().set_functions(Gdk.WMFunction.MOVE |
